#  ExchangeIO.pm - a FlowScan-derived class for reporting on traffic I/O
#  Copyright (C) 2002 Invisible Hand Networks, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# Based on SubNetIO.pm,v 1.27 2001/02/28 20:17:40 dplonka Exp 
# by Dave Plonka <plonka@doit.wisc.edu>
# This is $Id: ExchangeIO.pm,v 1.2 2002/07/14 07:45:40 nemo Exp $
# Nemo Semret, <nemo@invisiblehand.net>

use strict;

package ExchangeIO;

require 5;
require Exporter;

use CampusIO 1.053;

@ExchangeIO::ISA=qw(CampusIO Exporter);
# convert the RCS revision to a reasonable Exporter VERSION:
'$Revision: 1.2 $' =~ m/(\d+)\.(\d+)/ && (( $SubNetIO::VERSION ) = sprintf("%d.%03d", $1, $2));

=head1 NAME

ExchangeIO - a FlowScan module for reporting on ServerFarm or Exchange traffic I/O
by subnet and AS

=head1 SYNOPSIS

   $ flowscan ExchangeIO

or in F<flowscan.cf>:

   ReportClasses ExchangeIO

=head1 DESCRIPTION

ExchangeIO is a flowscan report for reporting on flows of traffic in and out of
specific subnets and ASes within a hosting, co-location, or exchange site.  It
is implemented as a class derived from CampusIO, so you run either CampusIO or
ExchangeIO, not both, since ExchangeIO inherits all the functionality of
CampusIO.  For instance, in your F<flowscan.cf>:

   ReportClasses ExchangeIO

To monitor an AS, 

- either add an "AS=" field to a record in a CampusIO LocalSubnetFile; the same
record may also have a "SUBNET=" field for CampusIO  to monitor it.
- or in an ExchangeIO ASFile. 

=head1 CONFIGURATION

C<ExchangeIO>'s configuration file is F<ExchangeIO.cf>.  This configuration
file is located in the directory in which the F<flowscan> script
resides.

The ExchangeIO configuration directives include:

=over 4

=item B<ASFiles>

This directive is optional.  It is a a comma-seperated list of files
containing the definitions of the ASes on which you'd like to
report (in addition to those given in CampusIO's LocalSubnetFile). 
E.g.:

   # ASFiles our_subnets.boulder
   ASFiles bin/our_ASes.boulder

=back

=cut

use Cflow qw(:flowvars 1.015); # for use in wanted sub
use RRDs;
use Boulder::Stream;
use IO::File;
use File::Basename;
use POSIX; # for mktime, strftime
use ConfigReader::DirectiveStyle;
use Net::Patricia 1.010;

my $c = new ConfigReader::DirectiveStyle;

$c->directive('ASFiles');
if (-f "${FindBin::Bin}/ExchangeIO.cf") {
  $c->load("${FindBin::Bin}/ExchangeIO.cf"); 
#  $ExchangeIO::ASFiles = $c->value('ASFiles');
}

# { initialize the Patricia Trie

# first read the ASes, if any,  in the LocalSubnetFiles of CampusIO
my($subnets_file, $stream, $cargo);
foreach $subnets_file (@CampusIO::subnets_files) {
   print(STDERR "ExchangeIO loading \"$subnets_file\" ...\n") if -t;
   my $fh = new IO::File "<$subnets_file";
   $fh || die "open \"$subnets_file\", \"r\": $!\n";
   $stream = new Boulder::Stream $fh;

   while ($cargo = $stream->read_record) {
      my $as = $cargo->get('AS');
      next unless $as;

      print(STDERR "ExchangeIO AS$as  ...\n") if -t;
      push(@ExchangeIO::ASes, $as);
   }

   undef $fh
}

# second read the ASes, if any,  in the ASFiles of ExchangeIO

if ($c->value('ASFiles')) {
  @ExchangeIO::AS_files = split(m/\s*,\s*/, $c->value('ASFiles'));
  @ExchangeIO::ases_files = <@ExchangeIO::AS_files>;

  my($ases_file, $stream, $cargo);
  foreach $ases_file (@ExchangeIO::ases_files) {
    print(STDERR "ExchangeIO loading \"$ases_file\" ...\n") if -t;
    my $fh = new IO::File "<$ases_file";
    $fh || die "open \"$ases_file\", \"r\": $!\n";
    $stream = new Boulder::Stream $fh;

   while ($cargo = $stream->read_record) {
      my $as = $cargo->get('AS');
      next unless $as;

      print(STDERR "ExchangeIO AS$as  ...\n") if -t;
      push(@ExchangeIO::ASes, $as);
   }

   undef $fh
  }
}
# }

sub new {
   my $self = {};
   my $class = shift;
   CampusIO::_init($self);
   return bless _init($self), $class
}

sub _init {
   my $self = shift;
   return $self
}

sub ASwanted {
   my $self = shift;
   my $ref = shift;
   $ref->{as}{$src_as}{out}{flows}++;
   $ref->{as}{$src_as}{out}{bytes} += $bytes;
   $ref->{as}{$src_as}{out}{pkts} += $pkts;
   $ref->{as}{$dst_as}{in}{flows}++;
   $ref->{as}{$dst_as}{in}{bytes} += $bytes;
   $ref->{as}{$dst_as}{in}{pkts} += $pkts;
}

sub report_AS {
    my $self = shift;
    my $as;

    foreach $as (@ExchangeIO::ASes) {
	my $file = $self->{outputdir} . "/AS$as.rrd";
	$file =~ s/\:/_/g;
	$self->createRRD($file, 'in_', 'out_') unless -f $file;
       
	my @values = ();
	my $thingy;
	foreach $thingy ('bytes', 'pkts', 'flows') {
	    push(@values, 
		 0+$self->{total}->{as}{$as}{in}{$thingy},
		 0+$self->{total}->{as}{$as}{out}{$thingy}
		 );
	}
	$self->updateRRD($file, @values);
   }
}

sub report {
   my $self = shift;
   my $as;

   $self->SUPER::report;
   $self->report_AS;
}

sub DESTROY {
    my $self = shift;
    my $as;

    foreach $as ($ExchangeIO::ASes) {
	delete $as->{in};
	delete $as->{out};
    }
    $self->SUPER::DESTROY
}

=head1 BUGS

=head1 AUTHOR

Nemo Semret <nemo@invisiblehand.net>

Copyright (C) 2002 Nemo Semret.
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

=head1 VERSION

The version number is the module file RCS revision number (B<$Revision: 1.2 $>)
with the minor number printed right justified with leading zeroes to 3
decimal places.  For instance, RCS revision 1.1 would yield a package version
number of 1.001.

This is so that revision 1.10 (which is version 1.010), for example, will
test greater than revision 1.2 (which is version 1.002) when you want to
B<require> a minimum version of this module.

=cut

1
